""" Two api endpoints are currently planned

    * GET /tutor/:id to retrive information about some tutor
    * POST /tutor/:username/:email create a new tutor and email password
    * GET /tutorlist list of all tutors with their scores
"""
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import (APIClient, APIRequestFactory, APITestCase,
                                 force_authenticate)

from core.models import Feedback, Tutor
from core.views import TutorApiViewSet
from util.factories import GradyUserFactory

NUMBER_OF_TUTORS = 3


class TutorListTests(APITestCase):

    @classmethod
    def setUpTestData(cls):
        cls.factory = APIRequestFactory()
        cls.user_factory = GradyUserFactory()

    def setUp(self):
        self.tutor_list = [self.user_factory.make_tutor()
                           for _ in range(NUMBER_OF_TUTORS)]
        self.reviewer = self.user_factory.make_reviewer()
        self.request = self.factory.get(reverse('tutor-list'))
        self.view = TutorApiViewSet.as_view({'get': 'list'})

        force_authenticate(self.request, user=self.reviewer.user)
        self.response = self.view(self.request)

    def test_can_access(self):
        self.assertEqual(self.response.status_code, status.HTTP_200_OK)

    def test_get_a_list_of_all_tutors(self):
        self.assertEqual(len(self.response.data), NUMBER_OF_TUTORS)

    def test_feedback_count_matches_database(self):
        def verify_fields(tutor_obj):
            t = Tutor.objects.get(user__username=tutor_obj['username'])
            return t.get_feedback_count() == tutor_obj['feedback_count']

        self.assertTrue(all(map(verify_fields, self.response.data)))

    def test_sum_of_feedback_count(self):
        self.assertEqual(sum(obj['feedback_count']
                             for obj in self.response.data),
                         Feedback.objects.count())


class TutorCreateTests(APITestCase):

    USERNAME = 'some weird name!'

    @classmethod
    def setUpTestData(cls):
        cls.factory = APIRequestFactory()
        cls.user_factory = GradyUserFactory()

    def setUp(self):
        self.reviewer = self.user_factory.make_reviewer()
        self.request = self.factory.post(reverse('tutor-list'),
                                         {'username': self.USERNAME})
        self.view = TutorApiViewSet.as_view({'post': 'create'})

        force_authenticate(self.request, user=self.reviewer.user)
        self.response = self.view(self.request, username=self.USERNAME)

    def test_can_access(self):
        self.assertEqual(self.response.status_code, status.HTTP_201_CREATED)

    def test_can_create(self):
        self.assertEqual(Tutor.objects.first().user.username, self.USERNAME)


class TutorDetailViewTests(APITestCase):

    @classmethod
    def setUpTestData(cls):
        cls.factory = APIClient()
        cls.user_factory = GradyUserFactory()

    def setUp(self):
        self.tutor = self.user_factory.make_tutor(username='fetterotto')
        self.reviewer = self.user_factory.make_reviewer()
        self.client = APIClient()
        self.client.force_authenticate(user=self.reviewer.user)

        url = reverse('tutor-detail', kwargs={'username': 'fetterotto'})
        self.response = self.client.get(url, format='json')

    def test_can_access(self):
        self.assertEqual(self.response.status_code, status.HTTP_200_OK)

    def test_can_view_tutor(self):
        self.assertEqual(self.response.data['username'],
                         self.tutor.user.username)
